@@ -51,7 +51,6 @@ import org.joni.ast.StringNode;
 import org.joni.constants.AnchorType;
 import org.joni.constants.EncloseType;
 import org.joni.constants.NodeType;
-import org.joni.constants.RegexState;
 import org.joni.constants.StackPopLevel;
 import org.joni.constants.TargetInfo;
 
@@ -62,8 +61,6 @@ final class Analyser extends Parser {
     }
 
     protected final void compile() {
-        regex.state = RegexState.COMPILING;
-
         if (Config.DEBUG) {
             Config.log.println(regex.encStringToString(bytes, getBegin(), getEnd()));
         }
@@ -173,13 +170,11 @@ final class Analyser extends Parser {
 
         if (Config.DEBUG_COMPILE) {
             if (Config.USE_NAMED_GROUP) Config.log.print(regex.nameTableToString());
-            Config.log.println("stack used: " + regex.stackNeeded);
+            Config.log.println("stack used: " + regex.requireStack);
             if (Config.USE_STRING_TEMPLATES) Config.log.print("templates: " + regex.templateNum + "\n");
             Config.log.println(new ByteCodePrinter(regex).byteCodeListToString());
 
         } // DEBUG_COMPILE
-
-        regex.state = RegexState.NORMAL;
     }
 
     private void noNameDisableMapFor_cosAlt(Node node, int[]map, Ptr counter) {
@@ -225,10 +220,10 @@ final class Analyser extends Parser {
     private void noNameDisableMapFor_anchor(Node node, int[]map, Ptr counter) {
         AnchorNode an = (AnchorNode)node;
         switch (an.type) {
-            case AnchorNode.PREC_READ:
-            case AnchorNode.PREC_READ_NOT:
-            case AnchorNode.LOOK_BEHIND:
-            case AnchorNode.LOOK_BEHIND_NOT:
+            case AnchorType.PREC_READ:
+            case AnchorType.PREC_READ_NOT:
+            case AnchorType.LOOK_BEHIND:
+            case AnchorType.LOOK_BEHIND_NOT:
                 an.setTarget(noNameDisableMap(an.target, map, counter));
         }
     }
@@ -267,6 +262,10 @@ final class Analyser extends Parser {
             break;
 
         case NodeType.ENCLOSE:
+            EncloseNode en = (EncloseNode)node;
+            if (en.type == EncloseType.CONDITION) {
+                en.regNum = map[en.regNum];
+            }
             renumberByMap(((EncloseNode)node).target, map);
             break;
 
@@ -383,6 +382,7 @@ final class Analyser extends Parser {
 
             case EncloseType.OPTION:
             case EncloseNode.STOP_BACKTRACK:
+            case EncloseNode.CONDITION:
                 info = quantifiersMemoryInfo(en.target);
                 break;
 
@@ -498,6 +498,7 @@ final class Analyser extends Parser {
 
             case EncloseType.OPTION:
             case EncloseType.STOP_BACKTRACK:
+            case EncloseNode.CONDITION:
                 min = getMinMatchLength(en.target);
                 break;
             } // inner switch
@@ -603,6 +604,7 @@ final class Analyser extends Parser {
 
             case EncloseType.OPTION:
             case EncloseType.STOP_BACKTRACK:
+            case EncloseNode.CONDITION:
                 max = getMaxMatchLength(en.target);
                 break;
             } // inner switch
@@ -715,6 +717,7 @@ final class Analyser extends Parser {
 
             case EncloseType.OPTION:
             case EncloseType.STOP_BACKTRACK:
+            case EncloseNode.CONDITION:
                 len = getCharLengthTree(en.target, level);
                 break;
             } // inner switch
@@ -938,6 +941,7 @@ final class Analyser extends Parser {
 
             case EncloseType.MEMORY:
             case EncloseType.STOP_BACKTRACK:
+            case EncloseNode.CONDITION:
                 n = getHeadValueNode(en.target, exact);
                 break;
             } // inner switch
@@ -1461,8 +1465,8 @@ final class Analyser extends Parser {
                     sn.set(sbuf, 0, sp);
                     return;
                 }
-                sp++;
             }
+            sp += len;
         }
     }
 
@@ -1890,6 +1894,14 @@ final class Analyser extends Parser {
                 }
                 break;
 
+            case EncloseNode.CONDITION:
+                if (Config.USE_NAMED_GROUP) {
+                    if (!en.isNameRef() && env.numNamed > 0 && syntax.captureOnlyNamedGroup() && !isCaptureGroup(env.option)) {
+                        newValueException(ERR_NUMBERED_BACKREF_OR_CALL_NOT_ALLOWED);
+                    }
+                }
+                setupTree(en.target, state);
+                break;
             } // inner switch
             break;
 
@@ -1912,7 +1924,7 @@ final class Analyser extends Parser {
                 break;
 
             case AnchorType.LOOK_BEHIND_NOT:
-                if (checkTypeTree(an.target, NodeType.ALLOWED_IN_LB, EncloseType.ALLOWED_IN_LB, AnchorType.ALLOWED_IN_LB)) newSyntaxException(ERR_INVALID_LOOK_BEHIND_PATTERN);
+                if (checkTypeTree(an.target, NodeType.ALLOWED_IN_LB, EncloseType.ALLOWED_IN_LB_NOT, AnchorType.ALLOWED_IN_LB_NOT)) newSyntaxException(ERR_INVALID_LOOK_BEHIND_PATTERN);
                 node = setupLookBehind(node);
                 if (node.getType() != NodeType.ANCHOR) continue restart;
                 setupTree(((AnchorNode)node).target, (state | IN_NOT));
@@ -2203,6 +2215,7 @@ final class Analyser extends Parser {
                 break;
 
             case EncloseType.STOP_BACKTRACK:
+            case EncloseType.CONDITION:
                 optimizeNodeLeft(en.target, opt, oenv);
                 break;
             } // inner switch
